/*
 * FILE      : lp_sm_lu.c
 * SYSTEM    : ktAό`Atsgđ^ako
 * FUNCTION  : sâktƃs{beBO
 * AUTHOR    : N.Tsuchiya  2000.02.28
 * REFFERENCE:Uwe H. Suhl, Leena M.Suhl, ORSA J. on Comp. Vol 2, No 4, Fall 1990 PP325-335,
              Computing Sparse LU Factorization for Large-Scale Linear Programming Bases
 */
static char	*__file =__FILE__;  /* should be copied into all .c files   */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>
#include <math.h>
#include "lp_ssc.h"
#include "lpvect3.h"
#include "lplufact.h"
#define		PIV_TOL 	0.01
#define		DROP_TOL	1e-15
//#define ISZERO_E(x,epsi)	(-(epsi)<=(x) && (x)<=(epsi))
//#define ISEQUAL_RE(x,y,epsi)	(WK2=FABS_MC(y),WK2=(WK2<1.0)?(epsi):WK2*(epsi),(x)-(y) <= WK2 && (x)-(y) >= -WK2)

#define DIAG_BUNRI    /* Ίpvft番āAt̑Ίpvf͂PŒɂ NT020605 */
#if defined(DIAG_BUNRI)  /* Ίpvft番āAt̑Ίpvf͂PŒɂ NT020605 */
#  define SKIP_EMPTY /* ̍sAN͂č NT020604 */
#endif
static LUMATRX	*glu;

/*	ID_b_BB_check_d	aa^߂Ă邩mF
/*******************************************************************/
int b_BB_check_d( int num, double *b0, double *b1, char *titl1, char *titl2 )
{
	int		i, nchk = lp_chk_, maxmsg = 10, cnt = 0;
	double	y1, epsi = 1e-7;

	for( i = 0; i < num; ++i ) {
		if( ISEQUAL_RE(b1[i], b0[i], epsi) ) continue;
		if( b0[i] != 0.0 ) {
			if( (y1 = b0[i] / b1[i]) < 0.0 ) y1 = - y1; /* y1 = |^Z| */
			if( y1 < 1.0 ) y1 = 1.0 / y1;
			y1 = y1 - 1.0;                /* NT010531 add if() */
			if( fabs(b0[i]) > 1.0 && ISZERO_E(y1, 1E-6) ) continue;
		} else y1 = b1[i];
		if( ++cnt < maxmsg )
			Dprintf("덷 [%d]%s(%.9g) %s(%.9g) (%e) -1(%e)\n", i, titl1, b0[i], titl2, b1[i], (b0[i] - b1[i]), y1);
	}
#ifdef _DEBUG_SSC
	CHKB(0, cnt, 0); /* */
	if(cnt > 0)Dprintf("b_BB_check_d num(%d) cnt(%d)\n", num, cnt); /*  */
#endif
	return cnt; /* nchk - chk_;*/
}
/*	ID_b_BB_check_v2	aa^߂Ă邩mF
/*******************************************************************/
int b_BB_check_v2( int num, VECT2 *b0, VECT2 *b1, char *titl1, char *titl2 )
{
	int		i, nchk = lp_chk_, maxmsg = 10, cnt = 0;
	double	y1, v0, v1, epsi = 1e-7;

	for( i = 0; i < num; ++i ) {
		if( b0->usef[i] != b0->nowf ) v0 = 0.0;
		else v0 = b0->va[i];
		if( b1->usef[i] != b1->nowf ) v1 = 0.0;
		else v1 = b1->va[i];
		if( ISEQUAL_RE(v1, v0, epsi) ) continue;
		if( v0 != 0.0 ) {
			if( (y1 = v0 / v1) < 0.0 ) y1 = - y1; /* y1 = |^Z| */
			if( y1 < 1.0 ) y1 = 1.0 / y1;
			y1 = y1 - 1.0;                /* NT010531 add if() */
			if( fabs(v0) > 1.0 && ISZERO_E(y1, 1E-6) ) continue;
		} else y1 = v1;
		if( ++cnt < maxmsg )
			Dprintf("덷 [%d]%s(%.9g) %s(%.9g) (%e) -1(%e)\n", i, titl1, v0, titl2, v1, (v0 - v1), y1);
	}
#ifdef _DEBUG_SSC
	CHKB(0, cnt, 0); /* */
	if(cnt > 0)Dprintf("b_BB_check_v2 num(%d) cnt(%d)\n", num, cnt); /*  */
#endif
	return cnt; /* nchk - chk_;*/
}

/*	ID_lu_kenzan	PʃxNgɂaa^|Čɖ߂邩mF
/*******************************************************************/
int lu_kenzan( LUFACT *b )
{
	int		j, rc = 0;
	/* PʃxNgɂaa^|Čɖ߂邩mF̂ */
	for( j = 0; j < b->sz; ++j ) {
		memset( lplu.dwk3, 0, b->sz*sizeof(double) );
		lplu.dwk3[j] = 1.0; /* PʃxNg̍쐬 */
		lu_ftran_d1( b, lplu.dwk3  );
		rc = lu_ftran_d2( b, lplu.dwk3  );
/*		if( rc > 0 ) break;  /* PłG[oړIB̂ŏI */
	}
	for( j = 0; j < b->sz; ++j ) {
		memset( lplu.dwk3, 0, b->sz*sizeof(double) );
		lplu.dwk3[j] = 1.0; /* PʃxNg̍쐬 */
		lu_btran_d1( b, lplu.dwk3, j  );
		rc = lu_btran_d2( b, lplu.dwk3  );
/*		if( rc > 0 ) break;  /* PłG[oړIB̂ŏI */
	}
	return rc;  /* G[o͐Ԃ */
}
/*	ID_lu_make_B_1	`FbNpɊŝP쐬  a|Čɖ߂邩mF̂ 
/*******************************************************************/
static int lu_make_B_1( LUMATRX *lu, VECT2 *vv, int j )
{
	int		rc = 0, i;
	double	dwk;

#ifdef _DEBUG_SSC
	if(lpdbgf[173])TRACEA(0), Dprintf("lu_make_B_1 j(%d) \n", j);
#endif
	lu->col_head[j] = -1;
/*	for( i = 0; i < lu->n_row; ++i ) { /*} */
	for( i = vv->top; i >= 0; i = vv->next[i] ) {
		dwk = vv->va[i];
		if( 0.0 == dwk ) continue;
		if( (rc = lumx_append( lu, i, j, dwk)) < 0 ) goto               quit;
	}
quit:
#ifdef _DEBUG_SSC
	if(lpdbgf[173])TRACEC(0), Dprintf("lu_make_B_1 rc(%d) \n", rc);
	if(lpdbgf[173]>1 ) lumx_dump_all( lu,"lu_make_B_1()Pus" );
#endif
	return rc;
}
/*	ID_lu_lu_append_tail	ό`xNĝPvf𖖔ɒǉ
/*******************************************************************/
static int lu_lu_append_tail( LUMATRX *lu, int aa,
	int typev,          /* 1:^(lw,up) 2:s^(br)  */
	int row, int col,   /* ^(lw,up)ł͍sA s^(br)ł͗As */
	double v, int *head, int *tail, PFVECT2 *aux )
{
	int		rc = 0, auxi;
	LUAVS *aav = &lu->av;
	                                CHKA(0, row, lu->n_row);
	aav->i[aa] = row;
	aav->j[aa] = col;
	aav->v[aa] = v;
	/* 僊NXV */
#if 1
	aav->nexti[aa] = -1;
	if( *tail < 0 )  *head = aa;
	else             aav->nexti[*tail] = aa;
	*tail = aa;
#else
	if( *head < 0 )  *tail = aa;
	aav->nexti[aa] = *head;
	*head = aa;
#endif
	/* NXV */
	if( typev == 1 ) auxi = row;  /* 1:^(lw,up) */
	else             auxi = col;  /* 2:s^(br)  */
	aav->nextj[aa] = aux[auxi].pf_head;
	aux[auxi].pf_head = aa;

#ifdef _DEBUG_SSC
	if(lpdbgf[167])TRACEB(0), Dprintf("lu_lu_append_tail rc(%d) \n", rc);
#endif
	return rc;
}
/*	ID_lu_uselink	ό`ŔΊpɔOvfC[^ϊ̃N
/*******************************************************************/
static int lu_uselink(SANKAKU *lwup, int which)
{
	int		i, pred, next;
	PFVECT	*wkpfv;
	PFVECT2	*wkaux;
	/* ()N */
	if( which & 0x01 ) {
		lwup->usebgnpfv = lwup->useendpfv = -1;
		pred = -1;
		for( i = lwup->allbgnpfv; i >= 0; i = next ) {
			wkpfv = &lwup->pfv[i]; /* ̂Ń|C^[Ngp */
			next = lwup->pfv[i].allnext;
/*			if( wkpfv->pf_head ) { /* ΊpɔOvf HY020625 */
			if( wkpfv->pf_head >= 0 ) { /* ΊpɔOvf */
				if( pred >= 0 ) lwup->pfv[pred].usenext = i;
				else            lwup->usebgnpfv = i;
				wkpfv->usepred = pred;
				pred = i;
			} else {
				wkpfv->usenext = wkpfv->usepred = -2;
			}
		}
		if( pred >= 0 ) lwup->pfv[pred].usenext = -1;
		else            lwup->usebgnpfv = -1;
		lwup->useendpfv = pred;
	}
	/* (s)N */
	if( which & 0x02 ) {
		lwup->usebgnaux = lwup->useendaux = -1;
		pred = -1;
		for( i = lwup->allbgnpfv; i >= 0; i = next ) {
			wkaux = &lwup->aux[i];
			next = lwup->pfv[i].allnext;
/*			if( wkaux->pf_head ) { /* ΊpɔOvf HY020625 */
			if( wkaux->pf_head >= 0 ) { /* ΊpɔOvf */
				if( pred >= 0 ) lwup->aux[pred].usenext = i;
				else            lwup->usebgnaux = i;
				wkaux->usepred = pred;
				pred = i;
			} else {
				wkaux->usenext = wkaux->usepred = -2;
			}
		}
		if( pred >= 0 ) lwup->aux[pred].usenext = -1;
		else            lwup->usebgnaux = -1;
		lwup->useendaux = pred;
	}
	return 0;
}
/*	ID_lu_make_eta	C[^ϊŐό`ɂâktts
/*******************************************************************/
static int lu_make_eta( LUFACT *b,
	int	inperfect,  /* 1:sS[h b_var[]<0LBϐȂƂɎg */
	int	rank)
{
	int		rc, k, row, col;
	int		bltail, butail, pa, aa, i, ii, nexti, skip_empty = 0;
	double	pivot, v;
	LUMATRX	*lu = b->lu;
	PFVECT	*bu;   /* [sz] */
	PFVECT	*bl;   /* [sz] */
	LUAVS	*aav;
	                                   CHKB(0, b->sz, b->x_sz);
	if( inperfect )                    CHKB(0, rank, b->sz);
	else                               CHKB(b->sz, rank, b->sz);
#ifdef _DEBUG_SSC
	if(lpdbgf[163])TRACEA(0), Dprintf("lu_make_eta sz(%d)\n", b->sz);
#endif
#if defined(SKIP_EMPTY) /* ̍sAN͂č NT020604 */
	skip_empty = 1;
#endif
	b->ll_nz = 0;     /* ksOvf̐BΊpvf    */
	b->uu_nz = 0;     /* tsOvf̐                   */
	/* kAqǏ邽߂ɁAʁXɍ */
	b->lw.typev = 1;  /* 1:^ 2:s^  */
	b->lw.allbgnpfv = b->lw.allendpfv = -1;    /* CHKB(0, b->n_pf, 0); */
	b->lw.usebgnpfv = b->lw.useendpfv = -1;
	b->up.typev = 1;  /* 1:^ 2:s^  */
	b->up.allbgnpfv = b->up.allendpfv = -1;
	b->up.usebgnpfv = b->up.useendpfv = -1;
	for( k = 0; k < b->sz; ++k ) {  /* HY020624 */
		b->lw.aux[k].pf_head = b->up.aux[k].pf_head = b->br.aux[k].pf_head = -1;
	}
	aav = &lu->av;
	for( k = 0; k < rank; ++k ) {
		row = lplu.mt[k][0];         CHKB(k, b->ips1[row], k);
		col = lplu.mt[k][1];         CHKB(k, b->ips2[col], k);
		if( inperfect && lplu.aibo[row] == -2 ) continue;  /* ϐԍ -2:݂Ȃ */
/* kAt𓯎ɍ쐬 HY020301 ==> */
		bl = &b->lw.pfv[row];
		bl->dim = row;
		bl->pf_head = bltail = -1;
		bu = &b->up.pfv[row];
		bu->dim = row;
		bu->pf_head = butail = -1;

		pivot = 0.0;
		pa = -1;               /* for check only */
		for( aa = lu->col_head[col]; aa >= 0; aa = nexti ) {
			i = aav->i[aa];
			ii = b->ips1[i];   /* ii:kts̍sԍ */
			nexti = aav->nexti[aa];
			/* t̍쐬BΊpvf(pivot)͋tɁB̗vf-1/pivotB */
			if( ii == k ) { pa = aa;   /* Ίpvf͂t */
				v = aav->v[aa];               CHKB(i, row, i);
				if( 0.0 == v ) { rc = -1; CHKA(k, i, -1); goto          quit; }
				pivot = 1.0 / v;       /* Ίpvf̋tL^Ă */
				b->diag_v[row] = pivot;
			} else if( ii < k ) { /* t̍쐬BΊpvf͕]BłP^Ίpvf|B */
				if( 0.0 == aav->v[aa] ) { CHKA(row, col, 0); continue; }  /*  NT000306 add */
				if( (rc = lu_lu_append_tail( lu, aa, 1, i, row, -aav->v[aa], &bu->pf_head, &butail, b->up.aux )) < 0 ) goto quit;
				++b->uu_nz;       /* tsOvf̐                   */
			} else {              /* k̍쐬BΊpvf͕] */
				if( 0.0 == aav->v[aa] ) { CHKA(k, i, 0); continue; } /*  NT000306 add */
				if( (rc = lu_lu_append_tail( lu, aa, 1, i, row, -aav->v[aa], &bl->pf_head, &bltail, b->lw.aux )) < 0 ) goto quit;
				++b->ll_nz;          /* ksOvf̐BΊpvf */
			}
		}                            CHKA(0, pa, lu->n_av);
		if( 0.0 == pivot ) { rc = -1; CHKA(b->sz, k, -1); goto          quit; }
		/* 僊N쐬FktcolOpsrowƂăN̖ɒǉ */
		bl->allpred = b->lw.allendpfv;
		bl->allnext = -1;
		if( b->lw.allendpfv < 0 ) {  b->lw.allbgnpfv  = row;
		} else { b->lw.pfv[b->lw.allendpfv].allnext = row;
		}
		b->lw.allendpfv = row;
		bu->allpred = b->up.allendpfv;
		bu->allnext = -1;
		if( b->up.allendpfv < 0 ) {  b->up.allbgnpfv  = row;
		} else { b->up.pfv[b->up.allendpfv].allnext = row;
		}
		b->up.allendpfv = row;
		/* N쐬FktcolOpsrowƂăN̖ɒǉ */
#if 0
		bl = &b->lw.aux[row];
		bu = &b->up.aux[row];
		bl->dim = row;
		bu->dim = row;
		bl->allpred = b->lw.allendaux;
		bl->allnext = -1;
		if( b->lw.allendaux < 0 ) {  b->lw.allbgnaux  = row;
		} else { b->lw.aux[b->lw.allendaux].allnext = row;
		}
		b->lw.allendaux = row;
		bu->allpred = b->up.allendaux;
		bu->allnext = -1;
		if( b->up.allendaux < 0 ) {  b->up.allbgnaux  = row;
		} else { b->up.aux[b->up.allendaux].allnext = row;
		}
		b->up.allendaux = row;
#endif
	}
	/* ΊpɔOvfC[^ϊ̃N */
	lu_uselink(&b->lw, 3);
	lu_uselink(&b->up, 3);
	rc = 0;
#ifdef _DEBUG_SSC
	/* col_head[]̃N͎gȂ */
	/* if(lpdbgf[163]>1) lumx_dump_all( b->lu, "C[^ϊs" ); */
#endif
quit:
#ifdef _DEBUG_SSC
	if(lpdbgf[163])TRACEC(0), Dprintf("lu_make_eta rc(%d) \n", rc);
#endif
	return rc;
}
/*	ID_nz_list_add	Ovfꗗ쐬
/*******************************************************************/
static int nz_list_add( LUMATRX	*lu, int crflg, int idx )
{
	int		pred, next, nzn, rc = 0;

	pred = -1;
	if( crflg == 0 ) {  CHKA(0, idx, lu->n_row); /* s̒ǉ */
		nzn = lu->row_nz[idx];  CHKB(1, nzn, lu->n_col); /* Ovf */
		if( nzn <= 0 ) goto                                             quit;
		next = lplu.r_nztop[nzn];
		if( next < 0 ) {  /* q[vɒǉ */
			rc = xheap_add( lplu.r_heap, nzn );
		} else {                CHKB(nzn, lplu.r_nzlst[next].nz, nzn);
			lplu.r_nzlst[next].pred = idx;
		}
		/* 擪ɒǉ */
		lplu.r_nztop[nzn] = idx;
		                        CHKB(-1, lplu.r_nzlst[idx].nz, -1);
		lplu.r_nzlst[idx].nz   = nzn;
		lplu.r_nzlst[idx].pred = pred;
		lplu.r_nzlst[idx].next = next;
	} else {            CHKA(0, idx, lu->n_col);
		nzn = lu->col_nz[idx];  CHKB(1, nzn, lu->n_row); /* Ovf */
		if( nzn <= 0 ) goto                                             quit;
		next = lplu.c_nztop[nzn];
		if( next < 0 ) {  /* q[vɒǉ */
			rc = xheap_add( lplu.c_heap, nzn );
		} else {                CHKB(nzn, lplu.c_nzlst[next].nz, nzn);
			lplu.c_nzlst[next].pred = idx;
		}
		/* 擪ɒǉ */
		lplu.c_nztop[nzn] = idx;
		                        CHKB(-1, lplu.c_nzlst[idx].nz, -1);
		lplu.c_nzlst[idx].nz   = nzn;
		lplu.c_nzlst[idx].pred = pred;
		lplu.c_nzlst[idx].next = next;
	}
quit:
	return( rc );
}
/*	ID_nz_list_del	Ovfꗗ폜
/*******************************************************************/
static int nz_list_del( LUMATRX	*lu, int crflg, int idx )
{
	int		pred, next, nzn, rc = 0;

	if( crflg == 0 ) { /* s̍폜 */  CHKA(0, idx, lu->n_row);
		nzn  = lplu.r_nzlst[idx].nz;   CHKB(1, nzn, lu->n_col);
		if( nzn < 0 ) goto                                              quit;
		pred = lplu.r_nzlst[idx].pred;
		next = lplu.r_nzlst[idx].next;
		if( pred >= 0 ) lplu.r_nzlst[pred].next = next;
		else			lplu.r_nztop[nzn] = next;
		if( next >= 0 ) lplu.r_nzlst[next].pred = pred;
		if( lplu.r_nztop[nzn] < 0 ) {  /* q[v폜 */
			rc = xheap_del( lplu.r_heap, nzn );
		}
		lplu.r_nzlst[idx].nz   = -1;
		lplu.r_nzlst[idx].pred = -1;
		lplu.r_nzlst[idx].next = -1;
	} else { /* ̍폜 */            CHKA(0, idx, lu->n_col);
		nzn  = lplu.c_nzlst[idx].nz;   CHKB(1, nzn, lu->n_row);
		if( nzn < 0 ) goto                                              quit;
		pred = lplu.c_nzlst[idx].pred;
		next = lplu.c_nzlst[idx].next;
		if( pred >= 0 ) lplu.c_nzlst[pred].next = next;
		else			lplu.c_nztop[nzn] = next;
		if( next >= 0 ) lplu.c_nzlst[next].pred = pred;
		if( lplu.c_nztop[nzn] < 0 ) {  /* q[v폜 */
			rc = xheap_del( lplu.c_heap, nzn );
		}
		lplu.c_nzlst[idx].nz   = -1;
		lplu.c_nzlst[idx].pred = -1;
		lplu.c_nzlst[idx].next = -1;
	}
quit:
	return( rc );
}
/*	ID_nz_list_chg	OvfύX
/*******************************************************************/
static int nz_list_change( LUMATRX	*lu, int crflg, int idx )
{
	int		nzn, rc = idx;

	if( crflg == 0 ) {  CHKA(0, idx, lu->n_row); /* s̔OvfύX */
		nzn  = lplu.r_nzlst[idx].nz; CHKB(0, nzn, lu->n_col);
		/* OvfύXȂ */
		if( nzn == lu->row_nz[idx] ) goto                               quit;
	} else {            CHKA(0, idx, lu->n_col); /* ̔OvfύX */
		nzn  = lplu.c_nzlst[idx].nz; CHKB(0, nzn, lu->n_row);
		/* OvfύXȂ */
		if( nzn == lu->col_nz[idx] ) goto                               quit;
	}
	rc = nz_list_del( lu, crflg, idx );  /* OvfN폜 */
	rc = nz_list_add( lu, crflg, idx );  /* VOvfNɑ} */
quit:
	return( rc );
}
/*	ID_nz_list_top	Ovf擪f[^܂́|PԂB擪͍폜 */
/*******************************************************************/
static int nz_list_top( LUMATRX	*lu, int crflg )
{
	int		idx, rc = 0;

	if( crflg == 0 ) { /* s */
		rc = xheap_peep( lplu.r_heap );
		if( rc < 0 ) goto                                               quit;
		idx = lplu.r_nztop[rc];       CHKA(0, idx, lu->n_row);
		                              CHKB(rc, lu->row_nz[idx], rc);
		                              CHKB(rc, lplu.r_nzlst[idx].nz, rc);
	} else {
		rc = xheap_peep( lplu.c_heap );
		if( rc < 0 ) goto                                               quit;
		idx = lplu.c_nztop[rc];       CHKA(0, idx, lu->n_col);
		                              CHKB(rc, lu->col_nz[idx], rc);
		                              CHKB(rc, lplu.c_nzlst[idx].nz, rc);
	}
	rc = nz_list_del( lu, crflg, idx );  /* OvfN폜 */
	rc = idx;
quit:
	return( rc );
}
static int comp_nz( int p, int q ) { return( p - q ); }
/*	ID_nz_list_renew	Ovfꗗ
/*******************************************************************/
static int nz_list_renew( int n_col, int n_row )
{
	int		i, j, rc = 0;

	if( (rc = xheap_renew(&lplu.c_heap, n_col, comp_nz)) < 0 ) goto     erra;
	if( (rc = xheap_renew(&lplu.r_heap, n_row, comp_nz)) < 0 ) goto     erra;
	for( i = 0; i < n_row; ++i ) {
		lplu.r_nztop[i]    = -1;
		lplu.r_nzlst[i].nz = -1;
	}
	for( j = 0; j < n_col; ++j ) {
		lplu.c_nztop[j]    = -1;
		lplu.c_nzlst[j].nz = -1;
	}
quit:
	return( rc );
erra:
	Eprintf( "**** b_renew: cannot malloc\n" );
	rc = MALLOC_ERROR;
	goto                                                                quit;
}

/*	ID_k_del_row	ktiks̗vf̍폜B폜̍s̗vfor-1Ԃ
/*******************************************************************/
static int k_del_row( LUMATRX *a, int pred, int aa,
	int add )  /* 1:{ԗpֈړ 0:폜̂ */
{
	int		row, rc = 0;
	LUAVS *aav = &a->av;

	row = aav->i[aa];
	if( pred < 0 ) {
		a->row_k_head[row] = aav->nextj[aa];
	} else {
		aav->nextj[pred] = aav->nextj[aa];        CHKB(row, aav->i[pred], row);
	}
	if( add ) {  /* {ԗpֈړ */
		aav->nextj[aa] = a->row_head[row];
		a->row_head[row] = aa;
	}
	return( rc );
}
/*	ID_k_del_col	ktiks̗vf̍폜B폜̍s̗vfor-1Ԃ
/*******************************************************************/
static int k_del_col( LUMATRX *a, int pred, int aa,
	int add )  /* 1:{ԗpֈړ 0:폜̂ */
{
	int		col, rc = 0;
	LUAVS *aav = &a->av;

	col = aav->j[aa];
	if( pred < 0 ) {
		a->col_k_head[col] = aav->nexti[aa];
	} else {
		aav->nexti[pred] = aav->nexti[aa];        CHKB(col, aav->j[pred], col);
	}
	if( add ) {  /* {ԗpֈړ */
		aav->nexti[aa] = a->col_head[col];
		a->col_head[col] = aa;
	}
	return( rc );
}
static int nz_check1( LUMATRX *lu, int aa )
{
	int		i, j, nz, chk_sav, ab;
	LUAVS	*aav = &lu->av;

	chk_sav = lp_chk_;
	i = aav->i[aa];
	j = aav->j[aa];
	nz = 0;
	for( ab = lu->col_k_head[j]; ab >= 0; ab = aav->nexti[ab] ) {
		++nz;
		if( aav->v[ab] == 0.0 )  CHKA(0, ab, ab);
	}                            CHKB(nz, lu->col_nz[j], nz);
	nz = 0;
	for( ab = lu->row_k_head[i]; ab >= 0; ab = aav->nextj[ab] ) {
		++nz;
		if( aav->v[ab] == 0.0 )  CHKA(0, ab, ab);
	}                            CHKB(nz, lu->row_nz[i], nz);
	return (lp_chk_ == chk_sav) ? 0 : -1;
}
static int nz_check_k( LUMATRX *lu, int sz )
{
	int		aa, k, nz, chk_sav;
	LUAVS	*aav = &lu->av;

	chk_sav = lp_chk_;
	for( k = 0; k < sz; ++k ) {
		nz = 0;
		for( aa = lu->col_k_head[k]; aa >= 0; aa = aav->nexti[aa] ) {
			++nz;
			if( aav->v[aa] == 0.0 )  CHKA(0, nz, nz);
		}                       CHKB(nz, lu->col_nz[k], nz);
		nz = 0;
		for( aa = lu->row_k_head[k]; aa >= 0; aa = aav->nextj[aa] ) {
			++nz;
			if( aav->v[aa] == 0.0 ) CHKA(0, nz, nz);
		}                       CHKB(nz, lu->row_nz[k], nz);
	}
	return (lp_chk_ == chk_sav) ? 0 : -1;
}		
#define IS_ZERO(x) ((x) == 0.0 ? 1 : 0 )
/*	ID_lu_change_pivot	kt̐vfwsI
/*******************************************************************/
static int lu_change_pivot( LUMATRX *lu, int k, /* i  */
	int *ips1, int *ips2, LUFACT *b)    /* io */
{
	int		aa, pa, rc, p, n1, n2, n_col, nrc, kc, kr, kk, row, col, nzcnt, pblk, ablk;
	int		predj, nextj, predi, nexti;
	double	pabs, avabs, x_off, stability, dkk1, v;
	double	pmkv, selm, amkv; /* I[ot[̂߁AdoubleϐɕύX NT000807(܏\Telɋ) */
	LUAVS	*aav = &lu->av;

	n1 = n2 = 0;                        /* ߂Xgƍs߂Xgɂ */
	n_col = lu->n_col;                  CHKB(n_col, lu->n_row, n_col);
	pa = -1;                        /* Is{bgvf */
	pmkv = (double)n_col * n_col;       /* Is{bgvfMarkowitz */
	pabs = 0.0;                         /* Is{bgvf̐Βl */
	pblk = 0;                           /* Is{bgvfubN 1: 0:O */
	nrc = 0;                            /* ςݍs{ */
	p  = 10;       /* pilot87Ŕj] */
	p  = 100;                           /* s{񐔂̍ŏlBȂƂ܂ł͒ */
	p  = 5;        /*  */
    /* [OvfP̍s܂͗AB̔Ovfs{bgɌ肷] */
	/* NT020524 ɒׂāAt̔O𑝉AkyBNT020524ɖ߂ */
	while( (row = nz_list_top( lu, 0 )) >= 0 ) { /* OPȏ̍s܂ŁAq[voB*/
		if( lu->row_nz[row] > 0 ) break;
		else { CHKA(row, lu->row_nz[row], 0); CHKB(-1, lu->row_k_head[row], -1); }
	}
	/* s݂AOvfPȂAB̔Ovfs{bgɌ肵āAI */
	if( row >= 0 ) {
		ips1[n1++] = row;
		if( lu->row_nz[row] == 1 ) {
			pa = lu->row_k_head[row];   CHKA(0, pa, lu->n_av);
			/* O܂łɏς܂͂OɂȂHvf̉N폜ȂALȃs{bgvfT */
			if( b->ips2[aav->j[pa]] < k || IS_ZERO(aav->v[pa]) ) {
				predj = -1;
				for( ; pa >= 0; pa = nextj ) {
					nextj = aav->nextj[pa];
					if( IS_ZERO(aav->v[pa]) ) {
						rc = k_del_row( lu, predj, pa, 0 );
						continue;
					}
					if( b->ips2[aav->j[pa]] > k ) break; /* vfꂽs{bgɌ肷 */
					rc = k_del_row( lu, predj, pa, 1 );
				}    
			}      CHKA(0, pa, lu->n_av);
			goto                                                        fin;
		}
	}
	while( (col = nz_list_top( lu, 1 )) >= 0 ) { /* OPȏ̗񂪌܂ŁAq[voB*/
		if( lu->col_nz[col] > 0 ) break;
		else { CHKA(col, lu->col_nz[col], 0); CHKB(-1, lu->col_k_head[col], -1); }
	}
	/* 񂪑݂AOvfPȂAB̔Ovfs{bgɌ肵āAI */
	if( col >= 0 ) {
		ips2[n2++] = col;
		if( lu->col_nz[col] == 1 ) {
			pa = lu->col_k_head[col];   CHKA(0, pa, lu->n_av);
			predi = -1;
			/* O܂łɏς܂͂OɂȂHvf̉N폜ȂALȃs{bgvfT */
			if( b->ips1[aav->i[pa]] < k || IS_ZERO(aav->v[pa]) ) {
				for( ; pa >= 0; pa = nexti ) {
					nexti = aav->nexti[pa];
					if( IS_ZERO(aav->v[pa]) ) {
						rc = k_del_col( lu, predi, pa, 0 );
						continue;
					}
					if( b->ips1[aav->i[pa]] > k ) break;
					rc = k_del_col( lu, predi, pa, 1 );
				}
			}      CHKA(0, pa, lu->n_av);
			goto                                                        fin;
		}
	}
	while( col >= 0 || row >= 0 ) { /* q[vɂȂ܂ŌJԂ */
		kr = (row < 0) ? n_col + 1 : lu->row_nz[row]; CHKA(0, 0, kr ); /* rows̔OBȂ炍{P*/
		kc = (col < 0) ? n_col + 1 : lu->col_nz[col]; CHKA(0, 0, kc ); /* col̔OBȂ炍{P*/
		kk = (kc < kr) ? kc : kr;
		dkk1 = (double)(kk - 1);
		/* I(|P)**2ƂAIȉ̃s{bgIĂAIցB*/
		selm = dkk1 * dkk1; /* I */
		if( pa >= 0 && pmkv <= selm ) goto                              fin;
/*		++nrc; */
		while( row >= 0 ) {
			if( lu->row_nz[row] > kk ) break;
			/* sőΒl̒ */
			if( lu->rmax_f[row] == 0 ) {
				x_off = 0.0;  /* s()̐Βl̍ől */
				predj = -1;
				for( aa = lu->row_k_head[row]; aa >= 0; aa = nextj ) {
					nextj = aav->nextj[aa];
					v = aav->v[aa];
					if( b->ips2[aav->j[aa]] < k ) {
						if( IS_ZERO(v) ) {   /* N͂č폜 NT020108*/
							rc = k_del_row( lu, predj, aa, 0 );
						} else {                  /* N͂Ĉړ */
							rc = k_del_row( lu, predj, aa, 1 );
						}
						continue;
					}
					avabs = (v < 0.0) ? -v : v;
					if( x_off < avabs ) x_off = avabs;                      /* őΒlXV */
					predj = aa;
				}
				lu->rmax_v[row] = x_off;         CHKX(0.0, 0.0, x_off);
				lu->rmax_f[row] = 1;
			} else {
				x_off = lu->rmax_v[row];  /* s()̐Βl̍ől */
			}
			stability = PIV_TOL * x_off;
			/* I @ AubN BMarkowitz CΒl */
			predj = -1;
			for( nzcnt = 0, aa = lu->row_k_head[row]; aa >= 0; aa = nextj, ++nzcnt ) {
				nextj = aav->nextj[aa];
				v = aav->v[aa];
				/* Ȃ͂̂bgjtAN͂č폜 */
				if( IS_ZERO(v) ) --nzcnt;
				/* Ȃ͂̂bgjtAN͂Ĉړ */
				if( b->ips2[aav->j[aa]] < k ) {  /* CHKB(k, k, b->ips2[aav->j]); AĊmFKv NT020108 */
					if( IS_ZERO(v) ) {   /* N͂č폜 NT020108*/
						rc = k_del_row( lu, predj, aa, 0 );
					} else {                  /* N͂Ĉړ */
						rc = k_del_row( lu, predj, aa, 1 );
					}
					continue;
				}
				predj = aa;
				amkv = dkk1 * (lu->col_nz[aav->j[aa]] - 1);    /* Markowitz  OP̏ꍇ͎OɒׂĂ */
/*NT020325		if( amkv <= 0 ) {  *pav = aav; goto fin; } IKvȂBrow_nz[]==0̎ɏIƂ܂ */
/*				if( amkv > pmkv ) continue; */
				avabs = (v < 0.0) ? -v : v;
				if( avabs <= stability ) continue; /* x𔻒 */
				ablk = 1;
				if( amkv > pmkv || amkv == pmkv && avabs <= pabs ) continue;
				pabs = avabs;  /* Markowitzŏ̒ŁAΒlőI */
				pmkv = amkv;
				pa = aa;
				pblk = ablk;
			}
			if( nzcnt != lu->row_nz[row] ) 
				CHKB(nzcnt, lu->row_nz[row], nzcnt);
			if( pmkv <= selm ) goto                                     fin;
			++nrc;
			if( nrc >= p && pmkv < (double)n_col * n_col ) goto         fin; 
			row = nz_list_top( lu, 0 );
			if( row >= 0 ) ips1[n1++] = row;
		}
		selm = kk * dkk1;
		if( pa >= 0 && pmkv <= selm ) goto                              fin;
		while( col >= 0 ) {
			if( lu->col_nz[col] > kk ) break;
			/* őΒl̒ */
			if( lu->cmax_f[col] == 0 ) {
				x_off = 0.0;  /* s()̐Βl̍ől */
				predi = -1;
				for( aa = lu->col_k_head[col]; aa >= 0; aa = nexti ) {
					nexti = aav->nexti[aa];
					v = aav->v[aa];
					if( b->ips1[aav->i[aa]] < k ) { /* cN͂Ĉړ */
						if( IS_ZERO(v) ) { /* cN͂č폜 */
							rc = k_del_col( lu, predi, aa, 0 );
						} else {
							rc = k_del_col( lu, predi, aa, 1 );
						}
						continue;
					}
					avabs = (v < 0.0) ? -v : v;
					if( x_off < avabs ) x_off = avabs;                      /* őΒlXV */
					predi = aa;
				}
				lu->cmax_v[col] = x_off;         CHKX(0.0, 0.0, x_off);
				lu->cmax_f[col] = 1;
			} else {
				x_off = lu->cmax_v[col];  /* s()̐Βl̍ől */
			}
			stability = PIV_TOL * x_off;
			/* I @ AubN BMarkowitz CΒl */
			predi = -1;
			for( nzcnt = 0, aa = lu->col_k_head[col]; aa >= 0; aa = nexti, ++nzcnt ) {
				nexti = aav->nexti[aa];
				v = aav->v[aa];
				/* Ȃ͂̂bgjtAcN͂č폜 */
				if( IS_ZERO(v) ) --nzcnt;
				/* Ȃ͂̂bgjtAcN͂Ĉړ */
				if( b->ips1[aav->i[aa]] < k ) { /*  CHKB(k, k, b->ips1[aav->i]); AĊmFKv NT020108 */
					if( IS_ZERO(v) ) { /* cN͂č폜 */
						rc = k_del_col( lu, predi, aa, 0 );
					} else {
						rc = k_del_col( lu, predi, aa, 1 );
					}
					continue;
				}
				predi = aa;
				amkv = dkk1 * (lu->row_nz[aav->i[aa]] - 1);  /* Markowitz OP̏ꍇ͎OɒׂĂ */
/*NT020325		if( amkv <= 0 ) { *pav = aav; goto  fin; } IKvȂBrow_nz[]==0̎ɏIƂ܂ */
/*				if( amkv > pmkv ) continue;*/
				avabs = (v < 0.0) ? -v : v;
				if( avabs <= stability ) continue; /* x𔻒 */
				ablk = 1;
				if( amkv > pmkv || amkv == pmkv && avabs <= pabs ) continue;
				pabs = avabs;
				pmkv = amkv;
				pa = aa;
				pblk = ablk;
			}                                    CHKB(nzcnt, lu->col_nz[col], nzcnt);
			if( pmkv <= selm ) goto                                     fin;
			++nrc;
			if( nrc >= p && pmkv < (double)n_col * n_col )goto          fin; 
			col = nz_list_top( lu, 1 );
			if( col >= 0 ) ips2[n2++] = col;
		}
	}
fin:                                    /* CHKA(0, pa, lu->n_av); */
	if( pa >= 0 ) {
		row = aav->i[pa];
		col = aav->j[pa];
	} else {    CHKA(0, 1, 1);
		row = col = -1;
	}
	for( kk = 0; kk < n1; ++kk ) { /* s{bgčsq[vɖ߂ */
		if( ips1[kk] == row ) { row = -1;
		} else if( ips1[kk] >= 0 ) { rc = nz_list_add( lu, 0, ips1[kk] );  CHKB(0, 0, rc);
		}
	}
	for( kk = 0; kk < n2; ++kk ) { /* s{bgėq[vɖ߂ */
		if( ips2[kk] == col ) { col = -1;
		} else if( ips2[kk] >= 0 ) { rc = nz_list_add( lu, 1, ips2[kk] );  CHKB(0, 0, rc);
		}
	}
	if( row >= 0 ) { rc = nz_list_del( lu, 0, row );  CHKB(0, 0, rc); }
	if( col >= 0 ) { rc = nz_list_del( lu, 1, col );  CHKB(0, 0, rc); }

	return pa;
}

/*	ID_makeUseAlpha	ԑ̂߂Ɋ֐
/*******************************************************************/
static void makeUseAlpha( LUMATRX *lu, int j )
{
	register int	*usef, nowf, aa, i;
	register LUAVS	*aav3 = &lu->av;
	register TYPE2  *t2;

	t2 = lplu.dalpha.pa;   /* NT020523 */
	usef = lplu.dalpha.usef;
	nowf = lplu.dalpha.nowf;
	for( aa = lu->col_k_head[j]; aa >= 0; aa = aav3->nexti[aa] ) {
		i = aav3->i[aa];
		t2[i].aa = aa;
		usef[i] = nowf;
	}
}

/*	ID_lu_fact	skt
/*******************************************************************/
static int lu_fact( LUFACT *b,
	int	inperfect,  /* 1:sS[h b_var[]<0LBϐȂƂɎg */
	int	*rank)
{
	int		rc = -1, i, j, k, row, col, max_col, err_cnt;
	int 	predi, predj, nexti, nextj, aik, akj, aa, pa;
	double	pivot, dikj, colv;
	LUMATRX	*lu = b->lu;
	LUAVS	*aav;
	int		*usef, nowf;
	TYPE2  *t2;

#ifdef _DEBUG_SSC
	if(lpdbgf[162])TRACEA(0), Dprintf("lu_fact \n");
#endif
	/* ԍips1[] ips2[]񐔁{Pŏ */
	max_col = -1;
	for( i = 0; i < b->sz; ++i ) {
		b->ips1[i] = b->ips2[i] = b->sz + 1;
/*		mpsort_lu( lu, lu->n_av, &lu->col_head[i], NULL ); */
	}
	/* OvfrL[Ƃėq[vƍsq[v */
	if( b->sz > 0 ) { /* TCYÔƂCHK邽 */
		if( (rc = nz_list_renew( lu->n_col+1, lu->n_row+1 )) < 0 ) goto quit;
	}
	for( i = 0; i < b->sz; ++i ) {
		if( lu->col_nz[i] > 0 ) rc = nz_list_add( lu, 1, i );
		if( lu->row_nz[i] > 0 ) rc = nz_list_add( lu, 0, i );
		b->lu->rmax_f[i] = 0;
		b->lu->cmax_f[i] = 0;
	}
	/* ktiks̏ */
	for( k = 0; k < b->sz; ++k ) {
		lu->col_k_head[k] = lu->col_head[k];
		lu->row_k_head[k] = lu->row_head[k];
		lu->col_head[k] = lu->row_head[k] = -1;
	}
	nz_check_k( lu, b->sz );  /* O`FbN */
	err_cnt = 10;
	aav = &lu->av;
	for( k = 0; k < b->sz ; ++k ) {
		/* ڍs琕ƂIBO̗vfcĂ͂lBO͍XVς */
/*		if( (rc = lu_change_pivot(lu, k, lplu.ips1, lplu.ips2, b, &pav)) < 0 ) break; */
		if( (pa = lu_change_pivot(lu, k, lplu.ips1, lplu.ips2, b)) < 0 ) break;
		row = aav->i[pa];                  CHKA(0, row, b->sz);
		col = aav->j[pa];                  CHKA(0, col, b->sz);
		lplu.mt[k][0] = row;
		lplu.mt[k][1] = col;           CHKB(k, b->ips1[row], b->ips1[row]);
		b->ips1[row] = k;              CHKB(k, b->ips2[col], b->ips2[col]);
		b->ips2[col] = k;
		if( 0.0 == aav->v[pa] )   {  CHKA(aav->i[pa], k, 0); break; }  /* NT020325 break */
/*		aav->v += aav->v_r; aav->v_r = 0.0; /* {xkt NT010823 */
		pivot = 1.0 / aav->v[pa]; 
		/* colrows(kts)艺vZ */
		predi = -1;
		for( aik = lu->col_k_head[col]; aik >= 0; aik = nexti ) {
			nexti = aav->nexti[aik];
			i = aav->i[aik];
			if( IS_ZERO(aav->v[aik]) ) { /* cN͂č폜 */
				rc = k_del_col( lu, predi, aik, 0 );
			} else if( b->ips1[i] <= k ) { /* cN͂ĈړBs{bgvf(ips1[i]=k)ňړ  */
				rc = k_del_col( lu, predi, aik, 1 );   /* b->ips1[i]:kts̍쐬 */
			} else {
/*				aav->v += aav->v_r; aav->v_r = 0.0; /* {xkt NT010823 */
				aav->v[aik] = aav->v[aik] * pivot;
				--lu->row_nz[i];  /* fill_invZ邽 */
				predi = aik;
			}
		}
/*		if( lu->row_k_head[row] ) goto                          nz_list; /* NT020523 add */
		/* ktiks̔폜iĚvZOj*/
		/* rowscolĚvZB akjrowsǂϐ */
		predj = -1;
		for( akj = lu->row_k_head[row]; akj >= 0; akj = nextj ) {
			nextj = aav->nextj[akj];
			j = aav->j[akj];
			if( IS_ZERO(aav->v[akj]) ) {  /* N͂č폜 */
				rc = k_del_row( lu, predj, akj, 0 );
				continue;
			} else if( b->ips2[j] <= k ) {  /* N͂ĈړBs{bgvf(ips2[i]=k)ňړ */
				rc = k_del_row( lu, predj, akj, 1 );
				continue;
			}
/*			aav->v += aav->v_r; aav->v_r = 0.0; /* {xkt NT010823 */
			predj = akj; 
			colv = aav->v[akj];
			aav->v[akj] = aav->v[akj] * pivot;
			--lu->col_nz[j];  /* fill_invZ邽 */
			if( lu->col_k_head[col] < 0 ) continue; /* NT020523 add */
			t2 = lplu.dalpha.pa;   /* NT020523 */
			usef = lplu.dalpha.usef;
			/* o */
			lplu.dalpha.n_elem = b->sz;   CHKB(1, b->sz, lplu.dalpha.x_elem); /* NT021029 */
			lpvect2_reset( &lplu.dalpha ); /* O|C^tzNA */
			nowf = lplu.dalpha.nowf;
#if 1
/*************************************************/
			predi = -1;
			for( aik = lu->col_k_head[j]; aik >= 0; aik = nexti ) {
				nexti = aav->nexti[aik];
				i = aav->i[aik];
				if( b->ips1[i] <= k ) {   /*CHKA(i, k, b->ips1[i]); /* b->ips1[i]:kts̍sԍB */
					/* cN͂A{NɈړ */
					if( IS_ZERO(aav->v[aik]) ) rc = k_del_col( lu, predi, aik, 0 );  /* N͂č폜 */
					else                       rc = k_del_col( lu, predi, aik, 1 );
					continue;
				}
				predi = aik;
				t2[i].aa = aik;
				usef[i] = nowf;
			}
/*************************************************/
#else
			makeUseAlpha( lu, j ); /* ԑ̂߂Ɋ֐ */
#endif
			/* aikcolǂϐ */
			predi = -1;
			for( aik = lu->col_k_head[col]; aik >= 0; aik = nexti ) {
				nexti = aav->nexti[aik];
				i = aav->i[aik];      /*̂CHKA(0, k, b->ips1[i]); /*  */
				/* ō폜Ă̂ŒʂȂ͂ */
				if( IS_ZERO(aav->v[aik]) )   CHKA(b->ips1[i], k, k);  /* cN͂č폜 */
				if( b->ips1[i] <= k )    CHKA(i, k, b->ips1[i]); /* b->ips1[i]:kts̍sԍB */
				dikj = aav->v[aik] * colv;
				if( usef[i] != nowf ) {          /* fill-invfǉ */
					if( (rc = lumx_k_append2(lu, i, j, -dikj )) < 0 ) goto quit;
				} else {
					aa = t2[i].aa;
					if( IS_ZERO(aav->v[aa]) ) {     /* OAvf̔O߂ */
						++lu->col_nz[aav->j[aa]];
						++lu->row_nz[aav->i[aa]];
						++lu->num_nz;
					}
					aav->v[aa] -= dikj;
					if( IS_ZERO(aav->v[aa]) ) {      /* OɂȂvf폜AOXV */
						--lu->col_nz[aav->j[aa]];
						--lu->row_nz[aav->i[aa]];
						--lu->num_nz;
					}
				}
				predi = aik;
			}
		}
		/* ύXsƗq[vŐʒuɈړ */
		for( akj = lu->row_k_head[row]; akj >= 0; akj = aav->nextj[akj] ) {
			j = aav->j[akj];     CHKA(0, k, b->ips2[j]); /*  */
			                     CHKB(1, lu->col_nz[j], lu->n_row);
			rc = nz_list_change( lu, 1, j );  CHKB(0, 0, rc);
			b->lu->cmax_f[j] = 0;
		}
		for( aik = lu->col_k_head[col]; aik >= 0; aik = aav->nexti[aik] ) {
			i = aav->i[aik];     CHKA(0, k, b->ips1[i]); /*  */
			                     CHKB(1, lu->row_nz[i], lu->n_col);
			rc = nz_list_change( lu, 0, i );  CHKB(0, 0, rc);
			b->lu->rmax_f[i] = 0;
		}
		/* ktiks̍폜iĚvZj*/
		lumx_k_rm_row_col( lu, row, col, 2);
	}
	*rank = k;
	/* ŌɎcvfړ */
	for( k = 0; k < b->sz; ++k ) {
		for( aik = lu->col_k_head[k]; aik >= 0; aik = nexti ) {
			nexti = aav->nexti[aik];
			if( IS_ZERO(aav->v[aik]) )  { CHKA(0, k, k); continue; }
			aav->nexti[aik] = lu->col_head[k];
			lu->col_head[k] = aik;
		}
		for( akj = lu->row_k_head[k]; akj >= 0; akj = nextj ) {
			nextj = aav->nextj[akj];
			if( IS_ZERO(aav->v[akj]) ) { CHKA(0, k, k); continue; }
			aav->nextj[akj] = lu->row_head[k];
			lu->row_head[k] = akj;
		}
		lu->col_k_head[k] = -1;
		lu->row_k_head[k] = -1;
	}
	for( ; k < b->sz ; ++k ) lplu.mt[k][0] = lplu.mt[k][1] = -1;
	if( *rank < b->sz && inperfect == 0 ) goto                          err2;
	rc = 0;
quit:
#ifdef _DEBUG_SSC
	if(lpdbgf[162])TRACEC(0), Dprintf("lu_fact rc(%d) \n", rc);
#endif
	return rc;
err2:
	Eprintf( "**** lu_fact: s{bgȂBTCY(%d)N(%d)\n", b->sz, *rank );
	rc = -1; goto                                                       quit;
}
/*	ID_lumx_copy	FTRAN,BTRANɂa|Čɖ߂邩mF̂ add NT110321
/*******************************************************************/
static int lumx_copy(
	LUMATRX	*bs,      //o
	LUMATRX	*lu)      //i
{
	int rc, k;

	bs->n_col = lu->n_col;              CHKB(1, bs->n_col, bs->x_col);
	bs->n_row = lu->n_row;              CHKB(1, bs->n_row, bs->x_row);
	if( (rc = lumx_reset( bs )) < 0 ) goto              quit; /* HY020225 */
	for( k = 0; k < lu->n_av; ++k ) {
		lumx_append( bs, lu->av.i[k], lu->av.j[k], lu->av.v[k] );
	}
	rc = 0;
quit:
	return rc;
}
/*	ID_lu_factorization	ktBSUHLv(?)ORSA J.C. Vol2-4 1990 PP325-335 fill-in팸kt
/*******************************************************************/
int lu_factorization( LUFACT *b,  /*  */
	int	inperfect,     /* 1:sS[h b_var[]<0LBϐȂƂɎg */
	int *rank,         /*  o */
	int **order,       /*  o ʂ̃f[^Yorder[i] */
	int **revord)      /*  o f[^̏ʂrevord[i] */
{
	int		rc = 0, i, row, col;
	                                     CHKB(0, 0, b->sz);
#ifdef _DEBUG_SSC
	if(lpdbgf[160])TRACEA(0), Dprintf("lu_factorization  \n");
	if(lpdbgf[160]>1 ) lumx_dump_all( b->lu,"Os" );
	if(lpdbgf[99]) lumx_copy(b->bs, b->lu); /* FTRAN,BTRANɂa|Čɖ߂邩mF̂ add NT110321 */
#endif
	b->lu_rfusoku = 0; /* 0: >0:Ns <0:̈s̃G[ NT020401 */
	*rank = 0;
	glu = b->lu;                         CHKB(glu->n_col, glu->n_row, glu->n_col);
	                                     CHKB(b->sz, glu->n_row, b->sz);
#if 0
	if( inperfect == 0 && b->sz > 0 ) {
#ifdef _DEBUG_SSC
		if(lpdbgf[160]>1)lumx_dump_all( b->lu,"s" );
#endif
		/* }b`O()Ō߂ktŎgvf̃Xg */
		if( (rc = b_make_mt(b, inperfect)) < 0 ) goto                           quit;
		/* ubNOp */
		rc = lp_block_sankaku(b->lu, b->sz, lplu.mt, &lplu.b3, inperfect );
#ifdef _DEBUG_SSC
/*		if(lpdbgf[160]>1) lu_dump_match(lplu.mt, b->sz, "ubNOp̃}b`O"); /* */
#endif
		/* sb.luɍ蒼 */
/*sv	if( (rc = sm_make_B( b, b->sz, b_var, inperfect)) < 0 ) goto quit; /* */
	} else { /* inperfect=1:sS[h b_var[]<0L */
		for( i = 0; i < b->sz; ++i ) {
			lplu.mt[i][0] = i;
			lplu.mt[i][1] = i;
			lplu.b3.list1[i] = i; /* s̏ubNԍBorsubN肷 */
			lplu.b3.list2[i] = i; /* 񂊂̏ubNԍBorsubN肷 */
		}
	}
#endif
	/* kt */
	if( (rc = lu_fact( b, inperfect, rank )) < 0 ) goto                 quit;
/*	if( b->u_pf < b->n_pf ) b->u_pf = b->n_pf; /* ̕Ŏgpő̐ NT010718 */
	/* C[^ϊŐό`ɂa̋ts߂ */
	if( (rc = lu_make_eta( b, inperfect, *rank )) < 0 ) goto            quit;
#if 0
	/* t̋N͂ NT020528 */
	int pred, next;
	pred = -1;
	for( i = b->up.allbgnpfv; i >= 0; i = next ) {
		next = b->up.pfv[i].next;
		if( b->up.pfv[i].pf_head >= 0 ) {
			pred = i;
		} else {
			b->up.pfv[i].next = -2;
			if( pred >= 0 ) b->up.pfv[pred].next = next;
			else            b->up.allbgnpfv = next;
		}
	}
#endif
	/* ϊ\쐬 */
	*order  = (int*)b->lu->row_k_head;
	*revord = (int*)b->lu->col_k_head;
	for( i = 0; i < b->sz; ++i ) {
		row = lplu.mt[i][0];           CHKA(0, row, b->sz);
		col = lplu.mt[i][1];           CHKA(0, col, b->sz);
		(*order)[row]  = col;      /* 񂉂ɂ̗͌order[i]ړ */
		(*revord)[col] = row;      /* ̗񂉂revord[i]Ɉړ */
		b->ips2[row] = i;          /* col̕Ԗlplu.ips2[row] */
		b->ips1[row] = col;        /* ussBftran,btranŕϐւpNT110322add */
	}
quit:
	if( rc < 0 ) b->lu_rfusoku = rc; /* 0: >0:Ns <0:̈s̃G[ NT020401 */
	else         b->lu_rfusoku = b->sz - *rank;
#ifdef _DEBUG_SSC
	if(lpdbgf[160]>1 || rc < 0 ){
/*		lumx_dump_all( b->lu,"s" ); col_head[]̃N͎gȂ */
		lu_dump_match(lplu.mt, b->sz, "kt̃}b`O");
		ssc_dump_int( b->ips1, b->sz, "b->ips1[]", 0 );
	    ssc_dump_int( b->ips2, b->sz, "b->ips2[]", 0 );
	    ssc_dump_double( b->diag_v, b->sz, "Ίp[]", 0 );
		lu_dump_L(b);
		lu_dump_U(b);
	}
	if(lpdbgf[160])TRACEC(0), Dprintf("lu_factorization rc(%d) rank(%d)\n", rc, *rank);
#endif
	return rc;
}

/*	ID_lu_pivotting	ts̍XVBinv(a)inv(ds)~inv(a)
/*******************************************************************/
int lu_pivotting( LUFACT *b, int r,  /* >0:(q̐)  0:덷̂ߐłȂȂ  <0:G[(̈s) */
	VECT2 *alpha2,  /* i  inv(k)~s */
	double *iB_r)   /*  o iB_rinv(a)̂sځBnull */
{
	int		rc = -1, j, apred, anext, abr, skip_empty = 0;
	double	heso, dwk, xx;
	int		pf_tail, aa, bb, pred;
	VECT2	*ur = &lplu.ur; /* */
	LUMATRX	*lu = b->lu;
	LUAVS	*aav;

#ifdef _DEBUG_SSC
	if(lpdbgf[161])TRACEA(0), Dprintf("lu_pivotting r(%d)\n", r);
	if(lpdbgf[161]>1)ssc_dump_vect2d( alpha2, b->sz, "s", 1 );
	if(lpdbgf[99])  /* `FbNpɊŝP쐬  a|Čɖ߂邩mF̂ */
		lu_make_B_1( b->bs, alpha2, r );
#endif
#if defined(SKIP_EMPTY) /* ̍sAN͂č NT020604 */
	skip_empty = 1;
#endif
	if( b->lu_rfusoku ) goto                                            err_rank;
	/* q(r~)̍쐬 */
	/*steepSDtsa^̂sڂ̍sxNgar߂Bardr~a^vZ(b_tran)B*/
	/* PʃxNgdr쐬 */
	ur->n_elem = b->sz; CHKB(1, b->sz, ur->x_elem);  /* NT021029 ƌoO */
	lpvect2_reset( ur );          /* O|C^tzNA */
	LPVECT2_APPEND( ur, r, 1.0 ); /* PʃxNgdr̍쐬 */
	/* rdr~invti (i=0..m-1)vZ */
	lu_btran_v21( b, ur, r );
	if(lpdbgf[161]>1)ssc_dump_vect2d( ur, b->sz, "r", 1 );
/*	memset( ur, 0, b->sz*sizeof(double) );
/*	ur[r] = 1.0; /* PʃxNgdr̍쐬 */
/*	lu_btran_d1( b, ur, r );         /* */

/*	if( iB_r ) {
/*		memcpy( iB_r, ur, sizeof(double)*b->sz );
/*		/* tϊ̌㔼B΁΁~q~invǩvZ */
/*		lu_btran_d2( b, iB_r );
/*	} */
	/* sinv(k)~sk^~švZBo͂kt̍s̓ꊷs */
	if( (rc = lu_ftran_v21( b, alpha2 )) < 0 ) goto                     quit;
	/* s'q~s */
/*	if( (rc = lu_row_eta_v2( alpha2, lu, b->br.pfv[abr].pf_head, b->br.pfv[abr].dim, &avr )) < 0 ) goto quit;*/
	/* ςvZ */
	xx = 0.0;
#if 1  /* ŒׂĂǂ炩ɌŒ肷 NT020603 */
	for( j = alpha2->top; j >= 0; j = alpha2->next[j] ) {
		if( ur->usef[j] == ur->nowf )
			xx += alpha2->va[j] * ur->va[j];
	}
#else
	for( j = ur->top; j >= 0; j = ur->next[j] ) {
		if( alpha2->usef[j] == alpha2->nowf )
			xx += alpha2->va[j] * ur->va[j];
	}
#endif
	LPVECT2_SET(alpha2, r, 1.0 ); /* pivottingłtm+1̃s{bgPɌŒ肷 */
	/* s'̂sOƔ񐳑ƂȂāAVts񂪂łȂ */
	if( ISZERO_E(xx, 1e-12) ) { /*CHKY(+1e-12, dwk, -1e-12); /* NT011218. KI020507 del CHK */
		rc = 0; goto                                                    quit;
	}
	/* (,r)qŕϊqƂĉɍ쐬(q̃JEgAbvȂ) */
	heso = 1.0 / xx;
	abr = b->n_br;                   CHKA(0, abr, b->x_br);
	b->br.pfv[abr].dim = r;
	b->br.pfv[abr].pf_head = -1;
	pf_tail = -1;
	for( j = ur->top; j >= 0; j = ur->next[j] ) {
		dwk = ur->va[j] * heso;             /* */
		if( ISZERO_E(dwk, 1e-30)  ) continue;	/* 2001.05.31 K.Igarashi */
		/* vfmۂ̂lumx_new_av()ĂԂ */
		if( (rc = lumx_only_new_av( lu )) < 0 ) goto                    quit;
		if( (rc = lu_lu_append_tail( lu, rc, 2, j, r, dwk, &b->br.pfv[abr].pf_head, &pf_tail, b->br.aux )) < 0 ) goto quit;
	}
#ifdef _DEBUG_SSC
	if(lpdbgf[161]>1){Dprintf("qŕϊq (%d)\n", r); lu_dump_lu( lu, b->br.pfv[abr].pf_head, r );}
#endif
	/* ɍ쐬(,r)qŕϊqƂĒǉ */
	b->n_br++;
	aav = &lu->av;
	/* t̂s̗vfN͂BOɂ */
	for( bb = b->up.aux[r].pf_head; bb >= 0; bb = aav->nextj[bb] ) {
		j = aav->j[bb];  /* ⏕N(s)ǂ */
		pred = -1; /* apf̑O̗vfBN͂ */
		for( aa = b->up.pfv[j].pf_head; aa >= 0; pred = aa, aa = aav->nexti[aa] ) {
			if( aav->i[aa] == r ) break;
		}
		if( aa >= 0 ) { /* t̂s̒lOɂ遁N͂ */
			if( pred < 0 ) b->up.pfv[j].pf_head = aav->nexti[aa];
			else           aav->nexti[pred] = aav->nexti[aa];
		} else CHKA(0, 0, 0);
		/* ɂȂftran btranpN͂ */
		if( skip_empty && b->up.pfv[j].pf_head < 0 && b->up.pfv[j].usenext != -2 ) {
			anext = b->up.pfv[j].usenext;
			apred = b->up.pfv[j].usepred;    CHKA(-1, b->up.pfv[j].usenext, b->sz);
			b->up.pfv[j].usenext = b->up.pfv[j].usepred = -2;
			if( apred < 0 ) { b->up.usebgnpfv = anext;
			} else {          b->up.pfv[apred].usenext = anext;
			}
			if( anext < 0 ) { b->up.useendpfv = apred;
			} else {          b->up.pfv[anext].usepred = apred;
			}
		}
	}
	/* t̗̂vfN͂BOɂ */
	for( bb = b->up.pfv[r].pf_head; bb >= 0; bb = aav->nexti[bb] ) {
		j = aav->i[bb];  /* 僊N()ǂ */
		pred = -1; /* apf̑O̗vfBN͂ */
		for( aa = b->up.aux[j].pf_head; aa >= 0 ; pred = aa, aa = aav->nextj[aa] ) {
			if( aav->j[aa] == r ) break;
		}
		if( aa >= 0 ) {
			if( pred >= 0 )  aav->nextj[pred] = aav->nextj[aa];
			else             b->up.aux[j].pf_head = aav->nextj[aa];
		} else CHKA(0, 0, 0);
	}
	b->up.aux[r].pf_head = -1;
	/* s'̑悒s𐕎Ƃŕϊq */
	/* 'tm+1Btm+1̗ԍ͂ƂBb->bu[r]ėp */
	b->up.pfv[r].pf_head = -1;
	pf_tail = -1;
	for( j = alpha2->top; j >= 0; j = alpha2->next[j] ) { CHKB(alpha2->nowf, alpha2->usef[j], alpha2->nowf);
		if( j != r ) {
			dwk = - alpha2->va[j];
			if( ISZERO_E(dwk, 1e-30) ) continue;	/* 2001.05.31 K.Igarashi */
			/* vfmۂ̂lumx_new_av()ĂԂ */
			if( (rc = lumx_only_new_av( lu )) < 0 ) goto                quit;
			if( (rc = lu_lu_append_tail( lu, rc, 1, j, r, dwk, &b->up.pfv[r].pf_head, &pf_tail, b->up.aux )) < 0 ) goto quit;
		}
	}
#ifdef _DEBUG_SSC
	if(lpdbgf[161]>1){Dprintf("s'̃ŕϊ (%d)\n", r); lu_dump_lu( lu, b->up.pfv[r].pf_head, r );}
#endif
	/* trN폜 */
	anext = b->up.pfv[r].allnext;
	apred = b->up.pfv[r].allpred;
	b->up.pfv[r].allnext = -2;
	if( apred < 0 ) { b->up.allbgnpfv = anext;
	} else {          b->up.pfv[apred].allnext = anext;
	}
	if( anext < 0 ) { b->up.allendpfv = apred;
	} else {          b->up.pfv[anext].allpred = apred;
	}
	if( b->up.pfv[r].usenext != -2 ) {  /* NT020528 */
		anext = b->up.pfv[r].usenext;
		apred = b->up.pfv[r].usepred;
		b->up.pfv[r].usenext = b->up.pfv[r].usepred = -2;
		if( apred < 0 ) { b->up.usebgnpfv = anext;
		} else {          b->up.pfv[apred].usenext = anext;
		}
		if( anext < 0 ) { b->up.useendpfv = apred;
		} else {          b->up.pfv[anext].usepred = apred;
		}
	}
	/* tm+1t̃N̖ɒǉ */
	apred = b->up.allendpfv;
	b->up.pfv[r].allnext = -1;
	b->up.pfv[r].allpred = apred;          CHKA(-1, apred, b->sz);
	if( apred < 0 ) { b->up.allbgnpfv = r;
	} else {          b->up.pfv[apred].allnext = r;
	}
	b->up.allendpfv = r;
	if( skip_empty && b->up.pfv[r].pf_head < 0 ) { CHKB(-2, b->up.pfv[r].usenext, -2);
		b->up.pfv[r].usenext = b->up.pfv[r].usepred = -2;
	} else {
		apred = b->up.useendpfv;
		b->up.pfv[r].usenext = -1;
		b->up.pfv[r].usepred = apred;          CHKA(-1, apred, b->sz);
		if( apred < 0 ) { b->up.usebgnpfv = r;
		} else {          b->up.pfv[apred].usenext = r;
		}
		b->up.useendpfv = r;
	}
#if 0
	/* tr⏕폜 */
	anext = b->up.aux[r].allnext;
	apred = b->up.aux[r].allpred;
	b->up.aux[r].allnext = -2;
	if( apred < 0 ) { b->up.allbgnaux = anext;
	} else {          b->up.aux[apred].allnext = anext;
	}
	if( anext < 0 ) { b->up.allendaux = apred;
	} else {          b->up.aux[anext].allpred = apred;
	}
	/* tm+1t̕⏕N̖ɒǉ */
	b->up.aux[r].allnext = -1;
	b->up.aux[r].allpred = b->up.allendaux;          CHKA(-1, b->up.allendaux, b->sz);
	if( b->up.allendaux < 0 ) { b->up.allbgnaux = r;
	} else {                b->up.aux[b->up.allendaux].allnext = r;
	}
	b->up.allendaux = r;
#endif
	/*  */
	/* ΊpɔOvfC[^ϊ̃N */
	lu_uselink(&b->up, 2);

	if(lpdbgf[99]>1) rc = lu_kenzan( b );
	rc = b->n_br;
quit:
#ifdef _DEBUG_SSC
	if(lpdbgf[161]>2) {
		ssc_dump_int( b->ips1, b->sz, "b->ips1[]", 0 );
	    ssc_dump_int( b->ips2, b->sz, "b->ips2[]", 0 );
	    ssc_dump_double( b->diag_v, b->sz, "Ίp[]", 0 );
		lu_dump_L(b);
		lu_dump_R(b);
		lu_dump_U(b);
	}
	if(lpdbgf[161])TRACEC(0), Dprintf("lu_pivotting rc(%d) \n", rc);
#endif
	return rc;
err_rank:                               CHKB(0, b->lu_rfusoku, 0);
	if( b->lu_rfusoku < 0 ) rc = b->lu_rfusoku;
	else                    rc = 0; /* 0:덷̂ߐłȂȂ  <0:G[(̈s) */
	goto                                                                quit;
}
/*	ID_lu_ftran_d1	ϊ̑OBq~k^~(ts̍XVŎg)
/*******************************************************************/
int lu_ftran_d1( LUFACT *b, double *vv )
{
	int		rc = 0, i, j, aa;
	register double	heso;   /* vv̂sڂ̒l */
	register double	ww;
	register PFVECT	*lw_pfv;
	LUMATRX	*lu = b->lu;
	register LUAVS *aav;
	
#ifdef _DEBUG_SSC
	if(lpdbgf[153])TRACEA(0), Dprintf("lu_ftran_d1 \n");
	if(lpdbgf[99])  /* a~a^~̃`FbNp */
		memcpy( lplu.dwk1, vv, b->sz*sizeof(double) );
#endif
	lw_pfv = b->lw.pfv;
	if( b->lu_rfusoku ) goto                                            err_rank;
	/* invki~ (i=0..m-1)vZ */
	/* \[XWJāCv */
	aav = &lu->av;
	for( j = b->lw.usebgnpfv; j >= 0; j = lw_pfv[j].usenext ) {
		if( 0.0 == (heso = vv[j]) ) { continue; }
		vv[j] = 0.0;  /* i'i{s~i ŌvZł悤ɂOɂĂ */
		/* heso = 0 ̎vv͕ωȂ͂Ȃ̂ŁAreturnėǂ͂ */
		/*	pa = -1;              /* ł̂ւOłȂ`FbNp */
		for( aa = lw_pfv[j].pf_head; aa >= 0; aa = aav->nexti[aa] ) {
			i = aav->i[aa];
			vv[i] += heso * aav->v[aa];       /* i'i{s~i if is */
		}                  CHKY(0.0, vv[j], 0.0);
		vv[j] = heso;  /* i'i{s~i ŌvZł悤ɂOɂĂ */
	}
	/* c~vZ */
	for( j = 0; j < b->sz; ++j )
		vv[j] *= b->diag_v[j];
	/* qi~ (i=0..)vZ */
	for( j = 0; j < b->n_br; ++j ) { /* us^Ł~u */
		/* \[XWJāCv */
		/* ςvZ */
		ww = 0.0;
		for( aa = b->br.pfv[j].pf_head; aa >= 0; aa = aav->nexti[aa] ) {
			i = aav->i[aa];
			ww += vv[i] * aav->v[aa];
		}
		vv[b->br.pfv[j].dim] = ww;
	}
	rc= 0;
quit:
#ifdef _DEBUG_SSC
	if(lpdbgf[153])TRACEC(0), Dprintf("lu_ftran_d1 rc(%d) \n", rc);
#endif
	return rc;
err_rank: rc = -1;                      CHKB(0, b->lu_rfusoku, 0);
	goto                                                                quit;
}
/*	ID_lu_ftran_d_T	ϊ̌㔼Bs~ (i=m-1..0)vZB
/*******************************************************************/
static void lu_ftran_d_T( LUFACT *b, double *vv ) /* lu_ftran_d1()ƋŃinv(a)~̌vZ */
{
	int k, j;
	for( k = 0; k < b->sz; ++k ) {
		j = b->ips1[k];
		lplu.dwk2[j] = vv[k];
	}
	for( k = 0; k < b->sz; ++k ) {
		vv[k] = lplu.dwk2[k];
	}
}

/*	ID_lu_ftran_d2	ϊ̌㔼Binvti~ (i=m-1..0)vZB
/*******************************************************************/
int lu_ftran_d2( LUFACT *b, double *vv ) /* lu_ftran_d1()ƋŃinv(a)~̌vZ */
{
	int		rc = 0, j, i, aa;
	register double	heso;   /* vv̂sڂ̒l */
	register PFVECT	*up_pfv;
	LUMATRX	*lu = b->lu;
	register LUAVS *aav;
	
#ifdef _DEBUG_SSC
	if(lpdbgf[153])TRACEA(0), Dprintf("lu_ftran_d2 \n");
#endif
	up_pfv = b->up.pfv;
	if( b->lu_rfusoku ) goto                                            err_rank;
	aav = &lu->av;
	/* invti~ (i=m-1..0)vZ */
	for( j = b->up.useendpfv; j >= 0; j = up_pfv[j].usepred ) {
		/* heso = 0 ̎vv͕ωȂ͂Ȃ̂ŁAreturnėǂ͂ */
		if( 0.0 == (heso = vv[j]) ) { continue; }
		vv[j] = heso;    /* t̑Ίpvf͂PŒ */
		for( aa = up_pfv[j].pf_head; aa >= 0; aa = aav->nexti[aa] ) {
			i = aav->i[aa];
			vv[i] += heso * aav->v[aa];       /* i'i{s~i if is */
		}
	}
	lu_ftran_d_T( b, vv );  //add NT110322
	rc= 0;          /* Z[h̎̓G[(b_BB_check_d̖߂l)Ԃ */
#ifdef _DEBUG_SSC
	if(lpdbgf[99]) { /* a~a^~̃`FbN */
		lumx_mult_mx_d( b->bs, lplu.dwk2, vv );
		rc = b_BB_check_d( b->bs->n_row, lplu.dwk1, lplu.dwk2, "(LU_ftrand)", "(aa^)" );
		if( rc && lpdbgf[250]>1){
			ssc_dump_double( lplu.dwk1, b->sz, "͂", 0 );
			ssc_dump_double( vv       , b->sz, "a^ ", 0 );
			ssc_dump_double( lplu.dwk2, b->sz, "Z", 0 );
		}
	}
#endif
quit:
#ifdef _DEBUG_SSC
	if(lpdbgf[153])TRACEC(0), Dprintf("lu_ftran_d2 rc(%d) \n", rc);
#endif
	return rc;
err_rank: rc = -1;                      CHKB(0, b->lu_rfusoku, 0);
	goto                                                                quit;
}

/*	ID_lu_btran_d_T	tϊ̑OB΁΁~švZB
/*******************************************************************/
static void lu_btran_d_T( LUFACT *b, double *vv ) /* lu_ftran_d1()ƋŃinv(a)~̌vZ */
{
	int k, j;
	for( k = 0; k < b->sz; ++k ) {
		j = b->ips1[k];
		lplu.dwk2[k] = vv[j];
//		lplu.dwk2[j] = vv[k];
	}
	for( k = 0; k < b->sz; ++k ) {
		vv[k] = lplu.dwk2[k];
	}
}
/*	ID_lu_btran_d1	tϊ̑OB΁΁~invťvZBOvf̗LԂ
/*******************************************************************/
int lu_btran_d1( LUFACT *b,
	double *vv,        /*  o vvarinv(a)̂s */
	int		e_col)     /* i  PʃxNĝP̈ʒuB-1:ʏxNg*/
/* vecPfListɂU~rsڂpf̃CfNXi[ 2001.06.13 K.Igarashi */
{
	int		rc = 0, j, i;
	register double	ww;
	register int aa;
	LUAVS *aav = &b->lu->av;
	int head;
#ifdef _DEBUG_SSC
	                                   CHKB(1, b->lw.typev, 1); CHKB(1, b->up.typev, 1);
	if(lpdbgf[154])TRACEA(0), Dprintf("lu_btran_d1 \n");
	if(lpdbgf[99]) /* a~a^~̃`FbNp */
		memcpy( lplu.dwk1, vv, b->sz*sizeof(double) );
#endif
	lu_btran_d_T( b, vv );  //add NT110322
	if( b->lu_rfusoku ) goto                                            err_rank;
	if( 1 ) { /* lu_btran_d2()Ƌɍłɂpilot87Ŕj]Bflg = 3020100003 0700000001 NT020218 */
		/* ΁΁~invti (i=0..m-1)vZ */
		for( j = b->up.usebgnpfv; j >= 0; j = b->up.pfv[j].usenext ) { /* uu~^ */
			/* ςvZ */
			ww = /*w_r =*/ 0.0;
			for( aa = b->up.pfv[j].pf_head; aa >= 0; aa = aav->nexti[aa] ) {
				i = aav->i[aa];
				ww += vv[i] * aav->v[aa];
			}
			ww += vv[j]; /* t̑Ίpvf͂PŒ */
			vv[j] = ww /*+ w_r*/;
/*			if( ww != 0.0 ) ++rc; */
		}
		rc = 1; /* NT020621 add */
	} else { /* łx݂ */
		/* Ovf̗L`FbN */
		if( e_col < 0 ) {
			head = b->up.usebgnaux;
			for( j = 0; j < b->sz; ++j ) {
				if( vv[j] == 0.0 ) continue;
				++rc; /* ̓Β̔Ovf */
				break;
			}
		} else {
			j = head = e_col;
			rc = 1;
		}
		/* ΁΁~invti (i=0..m-1)vZ */
		if( rc > 0 ) {
			for( j = head /*b->up.usebgnpfv*/; j >= 0; j = b->up.aux[j].usenext ) { /* uu~^ */
				if( vv[j] == 0.0 ) continue;
				ww = vv[j];
				/* švZ */
				/* Ίpvf̓NɓĂȂ */
				for( aa = b->up.aux[j].pf_head; aa >= 0; aa = aav->nextj[aa] ) {
					vv[aav->j[aa]] += ww * aav->v[aa];
				}
			}
		}
	}
quit:
#ifdef _DEBUG_SSC
	if(lpdbgf[154])TRACEC(0), Dprintf("lu_btran_d1 rc(%d) \n", rc);
#endif
	return rc;
err_rank: rc = -1;                      CHKB(0, b->lu_rfusoku, 0);
	goto                                                                quit;
}

/*	ID_lu_btran_d2	tϊ̌㔼B΁΁~q~invǩvZ
/*******************************************************************/
int lu_btran_d2( LUFACT *b,
	double *vv)         /*  o vvarinv(a)̂s */
{
	int		rc = 0, j, i, row, head, max_ips;
	register int aa;
	register double	heso;   /* vv̂sڂ̒l */
	register double	ww;
	LUAVS *aav = &b->lu->av;
#ifdef _DEBUG_SSC
	                                   CHKB(1, b->lw.typev, 1); CHKB(1, b->up.typev, 1);
	if(lpdbgf[154])TRACEA(0), Dprintf("lu_btran_d2 \n");
#endif
	if( b->lu_rfusoku ) goto                                            err_rank;
	/* ΁΁~qi (i=..0)vZ */
	for( j = b->n_br - 1; j >= 0 ; --j ) { /* uu~s^ */
		/*pf = b->pf;*/
		row = b->br.pfv[j].dim;
		heso = vv[row];
		if( 0.0 == heso ) { continue; }
		vv[row] = 0.0;  /* i'i{s~i ŌvZł悤ɂOɂĂ */
		/* heso = 0 ̎vv͕ωȂ͂Ȃ̂ŁAreturnėǂ͂ */
		for( aa = b->br.pfv[j].pf_head; aa >= 0; aa = aav->nexti[aa] ) {
			i = aav->i[aa];
			vv[i] += heso * aav->v[aa];       /* i'i{s~i if is */
		}
	}
	/* ΁΁~cvZ */
	for( j = 0; j < b->sz; ++j )
		vv[j] *= b->diag_v[j];
	/* ΁΁~invki (i=m-1..0)vZ */
	if( 1 ) {
		for( j = b->lw.useendpfv; j >= 0; j = b->lw.pfv[j].usepred ) {
			ww = vv[j];
			/* ςvZ */
			for( aa = b->lw.pfv[j].pf_head; aa >= 0; aa = aav->nexti[aa] ) {
				i = aav->i[aa];
				ww += vv[i] *aav->v[aa];
			}
			vv[j] = ww;
		}
	} else { /* łx݂ */
		/* OɑΊpvf݂̂|Ă(k͑Ίpvf̒l͂PȂ̂ŕsv) */
		head = b->lw.useendaux;
		max_ips = b->ips2[head];
		/* ΁΁~invki (i=m-1..0)vZ */
		if( max_ips >= 0 ) {
			for( j = head /*b->lw.useendpfv*/; j >= 0; j = b->lw.aux[j].usepred ) { /* uu~^ */
				ww = vv[j];
				if( ww == 0.0 ) continue; /* ISZERO_E葬̂Ŏcǂ */
				/* degen3ł͌ʏB15.4"==>15.0" pilot87ł͋t199"228"+CHK */
	/*			if( ISZERO_E(vv[j], 1e-10) ) { vv[j] = 0.0; continue; } /* */
	/*			if( ISZERO_E(vv[j], 1e-12) ) { vv[j] = 0.0; continue; } /* */
				if( ISZERO_E(ww, 1e-14) ) { vv[j] = 0.0; continue; } /* */
				/* švZ */
				for( aa = b->lw.aux[j].pf_head; aa >= 0; aa = aav->nextj[aa] ) {
					vv[aav->j[aa]] += ww * aav->v[aa];
				}
			}
		}
	}
	rc= 0;          /* Z[h̎̓G[(b_BB_check_d̖߂l)Ԃ */
#ifdef _DEBUG_SSC
	if(lpdbgf[99]) { /* ~a^~ã`FbN */
		lumx_mult_d_mx( b->bs, lplu.dwk2, vv );
		rc = b_BB_check_d( b->bs->n_row, lplu.dwk1, lplu.dwk2, "(LU_btrand)", "(΂a^a)" );
		if( rc && lpdbgf[250]>1){
			ssc_dump_double( lplu.dwk1, b->sz, "͂", 0 );
			ssc_dump_double( vv       , b->sz, "a^ ", 0 );
			ssc_dump_double( lplu.dwk2, b->sz, "Z", 0 );
		}
	}
#endif
quit:
#ifdef _DEBUG_SSC
	if(lpdbgf[154])TRACEC(0), Dprintf("lu_btran_d2 rc(%d) \n", rc);
	if(lpdbgf[154]>1)TRACEC(0), ssc_dump_double( vv, b->sz, " ", 0 );
#endif
	return rc;
err_rank: rc = -1;                      CHKB(0, b->lu_rfusoku, 0);
	goto                                                                quit;
}
/*=================================================*/

/*	ID_lu_ftran_v21	ϊ̑OBq~k^~(ts̍XVŎg)
/*******************************************************************/
int lu_ftran_v21( LUFACT *b, VECT2 *vv )
{
	int		rc = 0, j, i, head, pred, *usef, nowf;
	register int	aa;   /* vv̂sڂ̒l */
	register double	heso;   /* vv̂sڂ̒l */
	register double	ww, *pwk;
	register PFVECT	*lw_pfv;
	LUAVS	*aav = &b->lu->av;
	int		*vv_i, *vv_j, *vv_nexti, *vv_nextj;
	double	*vv_v;

	vv_i = aav->i;
	vv_j = aav->j;
	vv_nexti = aav->nexti;
	vv_nextj = aav->nextj;
	vv_v = aav->v;
	
#ifdef _DEBUG_SSC
	if(lpdbgf[155])TRACEA(0), Dprintf("lu_ftran_v21 \n");
	if(lpdbgf[99])  /* a~a^~̃`FbNp */
		memset( lplu.dwk1, 0, b->sz*sizeof(double) );
		for( j = vv->top; j >= 0; j = vv->next[j] ) { CHKA(0, j, b->sz);
			lplu.dwk1[j] = vv->va[j];
		}
#endif
	if( vv->top < 0 ) goto                                              quit;
	lw_pfv = b->lw.pfv;
	if( b->lu_rfusoku ) goto                                            err_rank;
	nowf = vv->nowf;
	usef = vv->usef;
	pwk  = vv->va;
	/* invki~ (i=0..m-1)vZ */
	if( vv->next[vv->top] >= 0 ) { /* vv̔OvfQȏ */
		head = b->lw.usebgnpfv;
	} else {                       /* vv̔OvfP */
		head = vv->top;
		if( lw_pfv[head].usenext == -2 ) head = -1;
	}
	for( j = head; j >= 0; j = lw_pfv[j].usenext ) {
		heso = (usef[j] == nowf) ? pwk[j] : 0.0;
		if( 0.0 == heso ) continue; /* heso = 0 ̎vv͕ωȂ */
		for( aa = lw_pfv[j].pf_head; aa >= 0; aa = vv_nexti[aa] ) {
			LPVECT2_ADD( vv, vv_i[aa], heso * vv_v[aa] );   /* i'i{s~i if is */
		}                    /* CHKY(0.0, pwk[j], 0.0); */
		pwk[j] = heso;  /* i'i{s~i ŌvZł悤ɂOɂĂ */
	}
	/* c~vZ */
	pred = -1;
	for( j = vv->top; j >= 0; j = vv->next[j] ) {
		if( pwk[j] != 0.0 ) {
			pwk[j] *= b->diag_v[j];
			pred = j;
		} else {
			usef[j] = 0;
			if( pred < 0 ) vv->top = vv->next[j];
			else           vv->next[pred] = vv->next[j];
		}
	}
	/* qi~ (i=0..)vZ */
	for( j = 0; j < b->n_br; ++j ) { /* us^Ł~u */
		/* ςvZ */
		ww = 0.0;
		for( aa = b->br.pfv[j].pf_head; aa >= 0; aa = vv_nexti[aa] ) {
			i = vv_i[aa];
			if( usef[i] == nowf ) {
				ww += pwk[i] * vv_v[aa];
			}
		}
		i = b->br.pfv[j].dim;
		if( ww != 0.0 )     LPVECT2_SET( vv, i, ww );
		else pwk[i] = 0.0;
	}
	rc= 0;
quit:
#ifdef _DEBUG_SSC
	if(lpdbgf[155])TRACEC(0), Dprintf("lu_ftran_v21 rc(%d) \n", rc);
#endif
	return rc;
err_rank: rc = -1;                      CHKB(0, b->lu_rfusoku, 0);
	goto                                                                quit;
}

/*	ID_lu_ftran_v22	ϊ̌㔼Binvti~ (i=m-1..0)vZB
/*******************************************************************/
int lu_ftran_v22( LUFACT *b, VECT2 *vv ) /* lu_ftran_v21()ƋŃinv(a)~̌vZ */
{
	int		rc = 0, j, head;
	register int	aa;
	register double	heso;   /* vv̂sڂ̒l */
	register PFVECT	*up_pfv;
	LUAVS	*aav = &b->lu->av;
	int		*vv_i, *vv_j, *vv_nexti, *vv_nextj;
	double	*vv_v;

	vv_i = aav->i;
	vv_j = aav->j;
	vv_nexti = aav->nexti;
	vv_nextj = aav->nextj;
	vv_v = aav->v;
	
#ifdef _DEBUG_SSC
	if(lpdbgf[155])TRACEA(0), Dprintf("lu_ftran_v22 \n");
#endif
	if( vv->top < 0 ) goto                                              quit;
	up_pfv = b->up.pfv;
	if( b->lu_rfusoku ) goto                                            err_rank;
	/* invti~ (i=m-1..0)vZ */
	if( vv->next[vv->top] >= 0 ) { /* vv̔OvfQȏ */
		head = b->up.useendpfv;
	} else {                       /* vv̔OvfP */
		head = vv->top;
		if( up_pfv[head].usenext == -2 ) { CHKB(-1, up_pfv[head].pf_head, -1);
			head = -1;
		}
	}
	for( j = head; j >= 0; j = up_pfv[j].usepred ) {
		heso = LPVECT2_GET(vv, j);
		if( 0.0 == heso ) continue; /* heso = 0 ̎vv͕ωȂ */
		for( aa = up_pfv[j].pf_head; aa >= 0; aa = vv_nexti[aa] ) {
			LPVECT2_ADD( vv, vv_i[aa], heso * vv_v[aa] );       /* i'i{s~i if is */
		}
	}
	rc= 0;          /* Z[h̎̓G[(b_BB_check_d̖߂l)Ԃ */
#ifdef _DEBUG_SSC
	if(lpdbgf[99]) { /* a~a^~̃`FbN */
		lumx_mult_mx_v2( b->bs, lplu.dwk2, vv );
		rc = b_BB_check_d( b->bs->n_row, lplu.dwk1, lplu.dwk2, "(LU_ftranv)", "(aa^)" );
		if( rc && lpdbgf[250]>1){
			ssc_dump_double( lplu.dwk1, b->sz, "͂", 0 );
			ssc_dump_double( vv->va   , b->sz, "a^ ", 0 );
			ssc_dump_double( lplu.dwk2, b->sz, "Z", 0 );
		}
	}
#endif
quit:
#ifdef _DEBUG_SSC
	if(lpdbgf[155])TRACEC(0), Dprintf("lu_ftran_v22 rc(%d) \n", rc);
	if(lpdbgf[155]>1)ssc_dump_double( vv->va, b->sz, " ", 0 );
#endif
	return rc;
err_rank: rc = -1;                      CHKB(0, b->lu_rfusoku, 0);
	goto                                                                quit;
}
/*	ID_lu_btran_v21	tϊ̑OB΁΁~invťvZ
/*******************************************************************/
int lu_btran_v21( LUFACT *b,
	register VECT2 *vv,        /*  o vvarinv(a)̂s */
	int		e_col)     /* i  PʃxNĝP̈ʒuB-1:ʏxNg */
/* vecPfListɂU~rsڂpf̃CfNXi[ 2001.06.13 K.Igarashi */
{
	int		rc = 0, i, head, jj;
	register int	j, *usef, nowf, aa;
	register double	ww, *pwk;
	register LUAVS	*aav = &b->lu->av;
	int		*vv_i, *vv_j, *vv_nexti, *vv_nextj;
	double	*vv_v;

	vv_i = aav->i;
	vv_j = aav->j;
	vv_nexti = aav->nexti;
	vv_nextj = aav->nextj;
	vv_v = aav->v;
#ifdef _DEBUG_SSC
	                                   CHKB(1, b->lw.typev, 1); CHKB(1, b->up.typev, 1);
	if(lpdbgf[156])TRACEA(0), Dprintf("lu_btran_v21 \n");
	if(lpdbgf[99]) { /* a~a^~̃`FbNp */
		memset( lplu.dwk1, 0, b->sz*sizeof(double) );
		for( j = vv->top; j >= 0; j = vv->next[j] ) { CHKA(0, j, b->sz);
			lplu.dwk1[j] = vv->va[j];
		}
	}
#endif
	if( vv->top < 0 ) goto                                              quit;
	if( b->lu_rfusoku ) goto                                            err_rank;
	nowf = vv->nowf;
	usef = vv->usef;
	pwk  = vv->va;
	if( 0 ) { /* ᑬ */
		/* ΁΁~invti (i=0..m-1)vZ */
		for( j = b->up.usebgnpfv; j >= 0; j = b->up.pfv[j].usenext ) { /* uu~^ */
			/* ςvZ */
			ww = (usef[j] == nowf) ? pwk[j] : 0.0;
			for( aa = b->up.pfv[j].pf_head; aa >= 0; aa = vv_nexti[aa] ) {
				i = vv_i[aa];
				if( usef[i] == nowf )
					ww += pwk[i] * vv_v[aa];
			}
			if( ww != 0.0 ) LPVECT2_SET( vv, j, ww );
			else pwk[j] = 0.0;
		}
	} else { /* łx݂ */
		/* ΁΁~invti (i=0..m-1)vZ */
		/* OɑΊpvf݂̂|ĂB<= Ίpvf͂cɕ̂ŕsv */
		if( vv->next[vv->top] >= 0 ) { /* vv̔OvfQȏ */
			head = b->up.usebgnaux;
		} else {                       /* vv̔OvfP */
			j = head = vv->top;
			if( usef[j] != nowf ) CHKB(nowf, usef[j], nowf);
			if( b->up.aux[j].usenext == -2 ) { CHKB(-1, b->up.aux[j].pf_head, -1);
				head = -1; 
			}
		}
		/* ΁΁~invti (i=0..m-1)vZ */
		for( j = head; j >= 0; j = b->up.aux[j].usenext ) { /* uu~^ */
			/* švZ */
			ww = (usef[j] == nowf) ? pwk[j] : 0.0;
			if( ww == 0.0 ) continue;
			for( aa = b->up.aux[j].pf_head; aa >= 0; aa = vv_nextj[aa] ) {
/*				LPVECT2_ADD( vv, aav->j, ww * aav->v ); */
				jj = vv_j[aa];
				if( usef[jj] == nowf ) {
					pwk[jj] += ww * vv_v[aa];
				} else {
					vv->next[jj] = vv->top;
					vv->top = jj;
					usef[jj] = nowf;
					pwk[jj]  = ww * vv_v[aa];
				}
			}
		}
	}
	rc = 0; /* 擪̂O폜ȂOvf̗L𒲍 NT020611 */
	for( j = vv->top; j >= 0; j = vv->next[j] ) {
		if( pwk[j] != 0.0 ) break;
		usef[j] = 0;
	}
	vv->top = j;
	if( j >= 0 ) rc = 1;
quit:
#ifdef _DEBUG_SSC
	if(lpdbgf[156])TRACEC(0), Dprintf("lu_btran_v21 rc(%d) \n", rc);
#endif
	return rc;
err_rank: rc = -1;                      CHKB(0, b->lu_rfusoku, 0);
	goto                                                                quit;
}
/*	ID_lu_btran_v22	tϊ̌㔼B΁΁~q~invǩvZ
/*******************************************************************/
int lu_btran_v22( LUFACT *b,
	VECT2 *vv)         /*  o vvarinv(a)̂s */
{
	int		rc = 0, j, i, row, pred, *usef, nowf, head, jj;
	register int	aa;
	register double	heso;   /* vv̂sڂ̒l */
	register double	ww, *pwk;
	register LUAVS *aav = &b->lu->av;
	int		*vv_i, *vv_j, *vv_nexti, *vv_nextj;
	double	*vv_v;

	vv_i = aav->i;
	vv_j = aav->j;
	vv_nexti = aav->nexti;
	vv_nextj = aav->nextj;
	vv_v = aav->v;
#ifdef _DEBUG_SSC
	                                   CHKB(1, b->lw.typev, 1); CHKB(1, b->up.typev, 1);
	if(lpdbgf[154])TRACEA(0), Dprintf("lu_btran_v22 \n");
#endif
	if( vv->top < 0 ) goto                                              quit;
	usef = vv->usef;
	nowf = vv->nowf;
	pwk  = vv->va;
	/* ΁΁~qi(i=..0)vZBqdimsĒPʍs */
	for( j = b->n_br - 1; j >= 0 ; --j ) { /* uu~s^ */
		row = b->br.pfv[j].dim;
		/* heso = 0 ̎vv͕ωȂ͂Ȃ̂ŁAreturnėǂ͂ */
		heso = (usef[row] == nowf) ? pwk[row] : 0.0;
		if( 0.0 == heso ) continue;
		pwk[row] = 0.0;  /* i'i{s~i ŌvZł悤ɂOɂĂ */
		for( aa = b->br.pfv[j].pf_head; aa >= 0; aa = vv_nexti[aa] ) {
/*			LPVECT2_ADD( vv, aav->i, heso * aav->v );    /* i'i{s~i if is */
			i = vv_i[aa];
			if( usef[i] == nowf ) {
				pwk[i] += heso * vv_v[aa];
			} else {
				vv->next[i] = vv->top;
				vv->top = i;
				usef[i] = nowf;
				pwk[i]  = heso * vv_v[aa];
			}
		}
	}
	/* ΁΁~cvZ */
	pred = -1;
	for( j = vv->top; j >= 0; j = vv->next[j] ) {
		if( pwk[j] != 0.0 ) {
			pwk[j] *= b->diag_v[j];
			pred = j;
		} else {
			usef[j] = 0;
			if( pred < 0 ) vv->top = vv->next[j];
			else           vv->next[pred] = vv->next[j];
		}
	}
	/* ΁΁~invki (i=m-1..0)vZ */
	if( 0 ) { /* ᑬ */
		for( j = b->lw.useendpfv; j >= 0; j = b->lw.pfv[j].usepred ) {
			/* ςvZ */
			ww = (usef[j] == nowf) ? pwk[j] : 0.0;  /* HY020514 */
			for( aa = b->lw.pfv[j].pf_head; aa >= 0; aa = vv_nexti[aa] ) {
				i = vv_i[aa];
				if( usef[i] == nowf )
					ww += pwk[i] * vv_v[aa];
			}
			if( ww != 0.0 ) LPVECT2_SET( vv, j, ww );
			else pwk[j] = 0.0;
		}
	} else {
		/* OɑΊpvf݂̂|Ă(k͑Ίpvf̒l͂PȂ̂ŕsv)HY020514 */
		if( vv->next[vv->top] >= 0 ) { /* vv̔OvfQȏ */
			head = b->lw.useendaux;
		} else {                       /* vv̔OvfP */
			j = head = vv->top;
			if( usef[j] != nowf ) CHKB(nowf, usef[j], nowf);
			if( b->lw.aux[j].usenext == -2 ) { CHKB(-1, b->lw.aux[j].pf_head, -1);
				head = -1;
			}
		}
		/* ΁΁~invki (i=m-1..0)vZ */
		for( j = head; j >= 0; j = b->lw.aux[j].usepred ) { /* uu~^ */
			/* švZ */
			ww = (usef[j] == nowf) ? pwk[j] : 0.0;
			if( ww == 0.0 )  continue;
			for( aa = b->lw.aux[j].pf_head; aa >= 0; aa = vv_nextj[aa] ) {
				jj = vv_j[aa];
/*				LPVECT2_ADD( vv, aav->j, ww * aav->v ); */
				if( usef[jj] == nowf ) {
					pwk[jj] += ww * vv_v[aa];
				} else {
					vv->next[jj] = vv->top;
					vv->top = jj;
					usef[jj] = nowf;
					pwk[jj]  = ww * vv_v[aa];
				}
			}
		}
		rc= 0;          /* Z[h̎̓G[(b_BB_check_d̖߂l)Ԃ */
	}
#ifdef _DEBUG_SSC
	if(lpdbgf[99]) { /* ~a^~ã`FbN */
		lumx_mult_v2_mx( b->bs, lplu.dwk2, vv );
		rc = b_BB_check_d( b->bs->n_row, lplu.dwk1, lplu.dwk2, "(LU_btranv)", "(΂a^a)" );
		if( rc && lpdbgf[250]>1){
			ssc_dump_double( lplu.dwk1, b->sz, "͂", 0 );
			ssc_dump_double( vv->va   , b->sz, "a^ ", 0 );
			ssc_dump_double( lplu.dwk2, b->sz, "Z", 0 );
		}
	}
#endif
quit:
#ifdef _DEBUG_SSC
	if(lpdbgf[154])TRACEC(0), Dprintf("lu_btran_v22 rc(%d) \n", rc);
#endif
	return rc;
}

